/*
 * Copyright (C) 2018 P.Bernal-Polo
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* 
 * File:   main.cpp
 * Author: P.Bernal-Polo
 * 
 * Created on March 10, 2018, 18:48 PM
 */


#include "config.h"

#include "MEKFcO.h"
#include "MUKFcO.h"
#include "MEKFcRP.h"
#include "MUKFcRP.h"
#include "MEKFcMRP.h"
#include "MUKFcMRP.h"
#include "MEKFcRV.h"
#include "MUKFcRV.h"


double fm = 0.0;
double fs = 0.0;
long n = 0;
long Ntimes = 100;

//MUKFcO estimator = MUKFcO();
//MUKFcRP estimator = MUKFcRP();
//MUKFcMRP estimator = MUKFcMRP();
//MUKFcRV estimator = MUKFcRV();
//MEKFcO estimator = MEKFcO();
//MEKFcRP estimator = MEKFcRP();
//MEKFcMRP estimator = MEKFcMRP();
MEKFcRV estimator = MEKFcRV();


// the setup routine runs once when you press reset:
void setup() {
  // we initialize the serial
  Serial.begin( 115200 );
}


// the loop routine runs over and over again forever:
void loop() {
  
  n++;
  // now we test the computational cost of our estimator
  //  we define the necessary entities
  double dt = 1.0e-2;
  double v[3] = { 0.0 , 0.0 , 1.0 };
  double vm[3] = { 0.0 , 1.0 , 0.0 };
  double wm[3] = { 0.0 , 0.0 , 0.0 };
  //  we perform the computational cost evaluation
  long ti = micros();
  estimator.update( v , vm , wm , dt );
  long tf = micros();
  double f = 1.0e6/( tf - ti );
  double alpha = 1.0/n;
  fm = (1.0-alpha)*fm + alpha*f;
  fs = (1.0-alpha)*fs + alpha*f*f;
  if(  n % Ntimes  ==  0  ){
    Serial.print( "MKF  CU " );
    Serial.print( fm , 6 );
    Serial.print( " " );
    Serial.print( sqrt( fs - fm*fm ) , 6 );
    Serial.print( "\n" );
  }
  
}
